-- some constants
local MODE_DEFAULT		= 1
local MODE_BOSS			= 2
local MODE_TRASH		= 3
local MODE_ALWAYS		= 4
local MODE_STRING = {
	[1] = "default",
	[2] = "boss",
	[3] = "trash",
	[4] = "always",
}
local DEFAULT_CFG = {
	dbg			= false,
	enable		= true,
	fadetime	= 1,
	mode		= MODE_DEFAULT,
}


-- addon config
CF_Config = CF_Config or {}
local realm
local player
local config
local frames = {
	[1] = nil,					-- the current chat frame
	[2] = GeneralDockManager,	-- the chat tabs
	[3] = ChatFrameMenuButton,	-- the chat arrows
	[4] = QuickJoinToastButton, -- the social button (O)
}

-- combat status
local mouseover	= false
local combat	= false
local encounter	= false

-- print helpers
local function sysprint(msg)
	print("|cFF33FF33ChatFix|r: " .. msg)
end

local function dbgprint(msg)
	if config.dbg == true then
		print("|cFFFF0000ChatFix[DBG]|r: " .. msg)
	end
end

-- hide/show chat
local hidden
local function UpdateChat()
	-- check if addon is enabled
	if(config.enable == false) then return end

	-- update status
	local hide = false
	if mouseover == false then
		if config.mode == MODE_ALWAYS then
			hide = true
		elseif config.mode == MODE_DEFAULT and combat == true then
			hide = true
		elseif config.mode == MODE_BOSS and combat == true and encounter == true then
			hide = true
		elseif config.mode == MODE_TRASH and combat == true and encounter == false then
			hide = true
		end
	end

	-- check if chat state has changed
	if hidden == hide then return end
	hidden = hide

	-- update current chat frame and hide/show the frames
	frames[1] = SELECTED_CHAT_FRAME
	for _, frame in pairs(frames) do
		local alpha = frame:GetAlpha()
		if hidden == false then
			frame.Show = frame.Show_
			frame:Show()
			UIFrameFadeIn(frame, 0, 1, 1)
			dbgprint("showing frame `" .. frame:GetName() .. "`")
		else
			frame.Show_ = frame.Show
			frame.Show = function() end
			UIFrameFadeOut(frame, config.fadetime * alpha, alpha, 0)
			frame.fadeInfo.finishedArg1 = frame
			frame.fadeInfo.finishedFunc = frame.Hide
			dbgprint("hidding frame `" .. frame:GetName() .. "`")
		end
	end
end

-- callbacks
local function OnEvent(self, event, ...)
	if (event == "ADDON_LOADED") then
		-- load player config
		realm = GetRealmName()
		player = UnitName("player")
		if CF_Config[realm] == nil then
			CF_Config[realm] = {}
		end
		if CF_Config[realm][player] == nil then
			CF_Config[realm][player] = DEFAULT_CFG
		end
		config = CF_Config[realm][player]
		setmetatable(config, {__index = DEFAULT_CFG})

		-- set combat and encounter in case of a hot reload
		mouseover = false
		combat = UnitAffectingCombat("player")
		encounter = UnitExists("boss1")

		-- register events
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		self:RegisterEvent("PLAYER_REGEN_DISABLED")
		self:RegisterEvent("ENCOUNTER_START")
		self:RegisterEvent("ENCOUNTER_END")

	elseif event == "PLAYER_REGEN_DISABLED" then
		combat = true
	elseif event == "PLAYER_REGEN_ENABLED" then
		combat = false
	elseif event == "ENCOUNTER_START" then
		encounter = true
	elseif event == "ENCOUNTER_END" then
		encounter = false
	end

	UpdateChat()
end

local function OnEnter()
	mouseover = true
	UpdateChat()
end

local function OnLeave()
	mouseover = false
	UpdateChat()
end

-- cmd line
SLASH_CHATFIX1 = "/chatfix"
SLASH_CHATFIX2 = "/cf"
SlashCmdList.CHATFIX = function(msg)
	local arg
	local arglist = {}
	for arg in msg:gmatch("[^%s]+") do
		table.insert(arglist, arg)
	end

	if #arglist == 0 then
		sysprint("Type `/cf help` to see available commands.")
		sysprint("\n"
			.. "|cFF8888FFdebug|r: " .. (config.dbg and "on" or "off") .. "\n"
			.. "|cFF8888FFenabled|r: " .. (config.enable and "true" or "false") .. "\n"
			.. "|cFF8888FFfade time|r: " .. config.fadetime .. "s\n"
			.. "|cFF8888FFmode|r: " .. MODE_STRING[config.mode])

	elseif arglist[1] == "help" then
		sysprint("\n"
			.. "cmdline:\n"
			.. "    |cFF8888FF/cf|r [options]\n"
			.. "    |cFF8888FF/chatfix|r [options]\n"
			.. "options:\n"
			.. "    |cFF8888FFdbg|r - toggle debug mode\n"
			.. "    |cFF8888FFenable|r\n"
			.. "    |cFF8888FFdisable|r\n"
			.. "    |cFF8888FFfadetime <time>|r - change fade time in seconds\n"
			.. "    |cFF8888FFmode <mode>|r - set operation mode\n"
			.. "modes:\n"
			.. "    |cFF8888FFdefault|r - hide chat in combat\n"
			.. "    |cFF8888FFboss|r - hide chat on boss encounters only\n"
			.. "    |cFF8888FFtrash|r - hide chat on trash only\n"
			.. "    |cFF8888FFalways|r - hide chat always\n")

	elseif arglist[1] == "dbg" then
		config.dbg = not config.dbg
		sysprint("debug mode " ..
			(config.dbg and "enabled" or "disabled"))

	elseif arglist[1] == "enable" then
		config.enable = true
		sysprint("chat fix functionality enabled")

	elseif arglist[1] == "disable" then
		config.enable = false
		sysprint("chat fix functionality disabled")

	elseif arglist[1] == "fadetime" then
		config.fadetime = tonumber(arglist[2])
		sysprint("fade time set to " .. config.fadetime .. "s")

	elseif arglist[1] == "mode" then
		if tonumber(arglist[2]) == MODE_DEFAULT or arglist[2] == "default" then
			config.mode = MODE_DEFAULT
			sysprint("mode set to default")

		elseif tonumber(arglist[2]) == MODE_BOSS or arglist[2] == "boss" then
			config.mode = MODE_BOSS
			sysprint("mode set to boss only")

		elseif tonumber(arglist[2]) == MODE_TRASH or arglist[2] == "trash" then
			config.mode = MODE_TRASH
			sysprint("mode set to trash only")

		elseif tonumber(arglist[2]) == MODE_ALWAYS or arglist[2] == "always" then
			config.mode = MODE_ALWAYS
			sysprint("mode set to always")

		else
			sysprint("\n    |cFF8888FFmode|r: " .. MODE_STRING[config.mode])

		end
	else
		sysprint("invalid command! Type `/cf help` to see available commands.")

	end

	UpdateChat()
end


-- addon setup
local ChatFix = CreateFrame("Frame", "ChatFix")
ChatFix:RegisterEvent("ADDON_LOADED")
ChatFix:SetScript("OnEvent", OnEvent)
ChatFix:SetScript("OnEnter", OnEnter)
ChatFix:SetScript("OnLeave", OnLeave)
ChatFix:SetFrameStrata("BACKGROUND")
ChatFix:SetPoint("TOPLEFT", "ChatFrame1", "TOPLEFT")
ChatFix:SetPoint("BOTTOMRIGHT", "ChatFrame1", "BOTTOMRIGHT")
